use strict;
use warnings;
use diagnostics;
use Getopt::Long;
use File::Path;
use File::Basename;
use Math::MatrixReal;

my $path = dirname($0);

my ($inputFile,$lociNumber,$markerSnp,$estimatedDiseaseSnpMAF,$targetPrevalence,$headNumber,$dCopyNumber);
GetOptions (
  "input=s"      => \$inputFile,
  "nloci=i"      => \$lociNumber,
  "riskallele=s" => \$markerSnp,
  "variantMAF=s" => \$estimatedDiseaseSnpMAF,
  "prevalence=s" => \$targetPrevalence,
  "nselect=i"    => \$headNumber,
  "ntotalvar=i"  => \$dCopyNumber,
);
my @dMAF = split(/,/,$estimatedDiseaseSnpMAF);
for (my $i=0;$i<scalar(@dMAF);$i++) {
    $dMAF[$i] += 0;
}
$targetPrevalence += 0;

print "\n1) perl SampleSeq.pl -input $inputFile -nloci $lociNumber -riskallele $markerSnp -variantMAF $estimatedDiseaseSnpMAF -prevalence $targetPrevalence -nselect $headNumber -ntotalvar $dCopyNumber";

my $paramsFileTemp = "./expected.txt";
my $Rscript        = "$path/SampleSeq.r";
my @markerSnp_1or2 = split(/,/,$markerSnp);
my $markerSnp_1or2 = \@markerSnp_1or2;

# $p_A/a                       = \@[lociNumber]
# $p_Ga_Y0/Y1                  = \@[lociNumber]->\%{'AA' 'Aa|aA' 'aa'} # to suit S1.R, 'a' presencts marker no mater marker is minor or major
# $control/caseLociMarkerSnpGT = \@[control/caseNumber]->\@[lociNumber]
# $markerSnp_1or2              = \@[lociNumber]
my ($p_a,$p_A,$p_Ga_Y0,$p_Ga_Y1,$controlLociMarkerSnpGT,$caseLociMarkerSnpGT,$controlNumber,$caseNumber,$controlID,$caseID) = parseInputFile($inputFile,$markerSnp_1or2,$lociNumber);
&AlgorithmX($lociNumber,$controlNumber,$caseNumber,\@dMAF,$targetPrevalence,$paramsFileTemp,$markerSnp_1or2,$p_A,$p_a,$p_Ga_Y1,$p_Ga_Y0,$controlLociMarkerSnpGT,$caseLociMarkerSnpGT,$controlID,$caseID);

print "\n2) Rscript $Rscript $paramsFileTemp $headNumber $dCopyNumber\n";
system("Rscript $Rscript $paramsFileTemp $headNumber $dCopyNumber");

sub parseInputFile {
    my ($input,$markerSnp_1or2_pIF,$lociNumber_pIF) = @_;
    my (@p_a_pIF,@p_A_pIF,@p_Ga_Y0_pIF,@p_Ga_Y1_pIF,@controlLociMarkerSnpGT_pIF,@caseLociMarkerSnpGT_pIF,@controlNumber_pIF,@caseNumber_pIF);
    my (@caseID_pIF,@controlID_pIF);
    # Family ID
    # Individual ID
    # Paternal ID
    # Maternal ID
    # Sex (1=male; 2=female; other=unknown)
    # Phenotype
    # Genotype */* (lociNumber)
    # FAM001  1  0 0  1  2  A G  1 1  G C 
    # FAM001  2  0 0  1  2  A A  1 2  0 0 
    my ($i,$j);
    for ($i=0;$i<$lociNumber_pIF;$i++) {
        $p_a_pIF[$i]           = 0;
        $p_A_pIF[$i]           = 0;
        my (%Y0,%Y1);
        $Y0{'AA'}              = 0;
        $Y0{'Aa|aA'}           = 0;
        $Y0{'aa'}              = 0;
        $Y1{'AA'}              = 0;
        $Y1{'Aa|aA'}           = 0;
        $Y1{'aa'}              = 0;
        $p_Ga_Y0_pIF[$i]       = \%Y0;
        $p_Ga_Y1_pIF[$i]       = \%Y1;
        $controlNumber_pIF[$i] = 0;
        $caseNumber_pIF[$i]    = 0
    }
    
    my ($li,@li,$cc,$lociN,$igt);
    my $controls = 0;
    my $cases    = 0;
    open(F,$input) || die "$!";
    while ($li=<F>) {
        chomp $li;
        @li    = split(/\s+/,$li);
        $cc    = $li[5];
        $lociN = (scalar(@li)-6)/2;
        die "ERROR: lociNumber isn't consistant with input file: $input ($lociN != $lociNumber_pIF)\n" if ($lociN != $lociNumber_pIF);
        if ($cc == 0) {
            my @controlGT_pIF;
            for ($i=0;$i<$lociNumber_pIF;$i++) {
                $controlGT_pIF[$i] = "";
                $igt = $li[5+1+$i*2];
                if ($igt eq $markerSnp_1or2_pIF->[$i]) {
                    $controlGT_pIF[$i] .= 'a';
                    $p_a_pIF[$i]++;
                } elsif ($igt eq '0') {
                    $controlGT_pIF[$i] .= '-';
                } else {
                    $controlGT_pIF[$i] .= 'A';
                    $p_A_pIF[$i]++;
                }
                $igt = $li[5+2+$i*2];
                if ($igt eq $markerSnp_1or2_pIF->[$i]) {
                    $controlGT_pIF[$i] .= 'a';
                    $p_a_pIF[$i]++;
                } elsif ($igt eq '0') {
                    $controlGT_pIF[$i] .= '-';
                } else {
                    $controlGT_pIF[$i] .= 'A';
                    $p_A_pIF[$i]++;
                }
                $p_Ga_Y0_pIF[$i]->{'AA'}++    if ($controlGT_pIF[$i] eq 'AA');
                $p_Ga_Y0_pIF[$i]->{'Aa|aA'}++ if ($controlGT_pIF[$i] eq 'Aa');
                $p_Ga_Y0_pIF[$i]->{'Aa|aA'}++ if ($controlGT_pIF[$i] eq 'aA');
                $p_Ga_Y0_pIF[$i]->{'aa'}++    if ($controlGT_pIF[$i] eq 'aa');
                $controlNumber_pIF[$i]++      if ($controlGT_pIF[$i] ne '--');
            }
            $controlLociMarkerSnpGT_pIF[$controls] = \@controlGT_pIF;
            $controlID_pIF[$controls]              = $li[1];
            $controls++;
        }
        if ($cc == 1) {
            my @caseGT_pIF;
            for ($i=0;$i<$lociNumber_pIF;$i++) {
                $caseGT_pIF[$i] = "";
                $igt = $li[5+1+$i*2];
                if ($igt eq $markerSnp_1or2_pIF->[$i]) {
                    $caseGT_pIF[$i] .= 'a';
                    $p_a_pIF[$i]++;
                } elsif ($igt eq '0') {
                    $caseGT_pIF[$i] .= '-';
                } else {
                    $caseGT_pIF[$i] .= 'A';
                    $p_A_pIF[$i]++;
                }
                $igt = $li[5+2+$i*2];
                if ($igt eq $markerSnp_1or2_pIF->[$i]) {
                    $caseGT_pIF[$i] .= 'a';
                    $p_a_pIF[$i]++;
                } elsif ($igt eq '0') {
                    $caseGT_pIF[$i] .= '-';
                } else {
                    $caseGT_pIF[$i] .= 'A';
                    $p_A_pIF[$i]++;
                }
                $p_Ga_Y1_pIF[$i]->{'AA'}++    if ($caseGT_pIF[$i] eq 'AA');
                $p_Ga_Y1_pIF[$i]->{'Aa|aA'}++ if ($caseGT_pIF[$i] eq 'Aa');
                $p_Ga_Y1_pIF[$i]->{'Aa|aA'}++ if ($caseGT_pIF[$i] eq 'aA');
                $p_Ga_Y1_pIF[$i]->{'aa'}++    if ($caseGT_pIF[$i] eq 'aa');
                $caseNumber_pIF[$i]++         if ($caseGT_pIF[$i] ne '--');
            }
            $caseLociMarkerSnpGT_pIF[$cases] = \@caseGT_pIF;
            $caseID_pIF[$cases]              = $li[1];
            $cases++;
        }        
    }
    close F;
    
    print "\ncases: $cases\tcontrols: $controls";
    print "\ncaseNumber:";
    for ($i=0;$i<$lociNumber_pIF;$i++) {
        print " ",$caseNumber_pIF[$i];
    }
    print "\tcontrolNumber:";
    for ($i=0;$i<$lociNumber_pIF;$i++) {
        print " ",$controlNumber_pIF[$i];
    }
    print "\n";
#    print "caseLociMarderSnpGT\n";
    for ($i=0;$i<$cases;$i++) {
        for ($j=0;$j<$lociNumber_pIF;$j++) {
#            print $caseLociMarkerSnpGT_pIF[$i]->[$j]," ";
        }
#        print "\n";
    }
#    print "controlLociMarderSnpGT\n";
    for ($i=0;$i<$controls;$i++) {
        for ($j=0;$j<$lociNumber_pIF;$j++) {
#            print $controlLociMarkerSnpGT_pIF[$i]->[$j]," ";
        }
#        print "\n";
    }
    print "\n";
    for ($i=0;$i<$lociNumber_pIF;$i++) {
        $p_a_pIF[$i]                /= 2*($controlNumber_pIF[$i]+$caseNumber_pIF[$i]);
        $p_A_pIF[$i]                /= 2*($controlNumber_pIF[$i]+$caseNumber_pIF[$i]);
        print "locus",$i+1,":\n  (a)$p_a_pIF[$i]\t(A)$p_A_pIF[$i]\n";
        $p_Ga_Y0_pIF[$i]->{'AA'}    /= $controlNumber_pIF[$i];
        $p_Ga_Y0_pIF[$i]->{'Aa|aA'} /= $controlNumber_pIF[$i];
        $p_Ga_Y0_pIF[$i]->{'aa'}    /= $controlNumber_pIF[$i];
        $p_Ga_Y1_pIF[$i]->{'AA'}    /= $caseNumber_pIF[$i];
        $p_Ga_Y1_pIF[$i]->{'Aa|aA'} /= $caseNumber_pIF[$i];
        $p_Ga_Y1_pIF[$i]->{'aa'}    /= $caseNumber_pIF[$i];
        print "  pGaY0{AA}: ",$p_Ga_Y0_pIF[$i]->{'AA'},"\tpGaY0{Aa|aA}: ",$p_Ga_Y0_pIF[$i]->{'Aa|aA'},"\tpGaY0{aa}: ",$p_Ga_Y0_pIF[$i]->{'aa'},"\n";
        print "  pGaY1{AA}: ",$p_Ga_Y1_pIF[$i]->{'AA'},"\tpGaY1{Aa|aA}: ",$p_Ga_Y1_pIF[$i]->{'Aa|aA'},"\tpGaY1{aa}: ",$p_Ga_Y1_pIF[$i]->{'aa'},"\n";
    }
    print "\n";
    return(\@p_a_pIF,\@p_A_pIF,\@p_Ga_Y0_pIF,\@p_Ga_Y1_pIF,\@controlLociMarkerSnpGT_pIF,\@caseLociMarkerSnpGT_pIF,$controls,$cases,\@controlID_pIF,\@caseID_pIF);
}

sub AlgorithmX {
    my ($lociNumber_AX,$controlNumber_AX,$caseNumber_AX,$dMAF_AX,$targetPrevalence_AX,$paramsFileTemp_AX,$markerSnp_1or2_AX,$p_A_AX,$p_a_AX,$p_Ga_Y1_AX,$p_Ga_Y0_AX,$controlLociMarkerSnpGT_AX,$caseLociMarkerSnpGT_AX,$controlID_AX,$caseID_AX) = @_;
    my ($i,$j);
    my ($p_d_AX,$K);
    $p_d_AX = $dMAF_AX;
    $K      = $targetPrevalence_AX;   
    if   ($lociNumber_AX != scalar(@$p_d_AX)) { die "ERROR: lociNumber != the size of p_d array\n"; }
    if ( ($lociNumber_AX != scalar(@$p_A_AX))||($lociNumber_AX != scalar(@$p_a_AX)) ) { die "ERROR: lociNumber != the size of p_A/p_a array\n"; }
    # $p_dA/DA/Da/da = \@
    my ($p_dA,$p_DA,$p_Da,$p_da) = &calculateHaplotypeP($p_d_AX,$p_A_AX,$p_a_AX,$lociNumber_AX); # $p_d/A/a_AX = \@
       
    my (@array_E_Gd_GaY_hash,@array_missingData_E_Gd);
    for ($i=0;$i<$lociNumber_AX;$i++) {
        print "locus",$i+1,":\n";
        ($array_E_Gd_GaY_hash[$i],$array_missingData_E_Gd[$i]) = &calculateE_Gd_GaY($K,$p_Ga_Y0_AX->[$i],$p_Ga_Y1_AX->[$i],$p_dA->[$i],$p_DA->[$i],$p_Da->[$i],$p_da->[$i]);
    }
    
    my (@sample_ID);
    my (@sample_asum,@sample_EDsum);
    my (@sample_a,@sample_ED);
    my (@sampleMarkerGT);
    my (@sample_control0,@sample_case1,$sample_control0,$sample_case1);
    my ($sampleLociGaY,$sampleLociGaValue,$sampleLociMarkerGT);
    my ($E_Gd_GaY);
    my ($j_loci,$k_sample);
    $k_sample = 0;
     
    ####################### controls
    for ($i=0;$i<$controlNumber_AX;$i++) {
        $sample_asum[$k_sample]  = 0;
        $sample_EDsum[$k_sample] = 0;
        ($sampleLociGaY,$sampleLociGaValue,$sampleLociMarkerGT,$sample_control0,$sample_case1) = &getSampleLociGaY($controlLociMarkerSnpGT_AX->[$i],0);
        my (@sampleMarkerGT_each_locus,@sample_d_each_locus,@sample_a_each_locus,@sample_ED_each_locus);    
        for ($j_loci=0;$j_loci<scalar(@$sampleLociGaY);$j_loci++) {
            if ($sampleLociMarkerGT->[$j_loci] ne '--') {
                $E_Gd_GaY                     = $array_E_Gd_GaY_hash[$j_loci]->{$sampleLociGaY->[$j_loci]};
                $sample_asum[$k_sample]      += $sampleLociGaValue->[$j_loci];
                $sample_a_each_locus[$j_loci] = $sampleLociGaValue->[$j_loci];
            } else {
                $E_Gd_GaY                     = $array_missingData_E_Gd[$j_loci]->{'control'};
                $sample_asum[$k_sample]      += 0;
                $sample_a_each_locus[$j_loci] = '-';
            }
            $sample_EDsum[$k_sample]           += $E_Gd_GaY;
            $sampleMarkerGT_each_locus[$j_loci] = $sampleLociMarkerGT->[$j_loci];
            $sample_ED_each_locus[$j_loci]      = $E_Gd_GaY;
            #print "    j_loci: $j_loci\t$sampleLociGaY->[$j_loci]\t $E_Gd_GaY\n";
        }
        $sample_control0[$k_sample] = $sample_control0;
        $sample_case1[$k_sample]    = $sample_case1;
        $sampleMarkerGT[$k_sample]  = \@sampleMarkerGT_each_locus;
        $sample_a[$k_sample]        = \@sample_a_each_locus;
        $sample_ED[$k_sample]       = \@sample_ED_each_locus;
        $sample_ID[$k_sample]       = $controlID_AX->[$i];
        #print "  dataset ",$i_dataset+1," (individul ",$k_sample+1,") ED: $sample_EDsum[$k_sample] D: $sample_dsum[$k_sample]\n";
        $k_sample++;
    }
    ####################### cases
    for ($i=0;$i<$caseNumber_AX;$i++) {
        $sample_asum[$k_sample]  = 0;
        $sample_EDsum[$k_sample] = 0;
        ($sampleLociGaY,$sampleLociGaValue,$sampleLociMarkerGT,$sample_control0,$sample_case1) = &getSampleLociGaY($caseLociMarkerSnpGT_AX->[$i],1);
        my (@sampleMarkerGT_each_locus,@sample_a_each_locus,@sample_ED_each_locus);    
        for ($j_loci=0;$j_loci<scalar(@$sampleLociGaY);$j_loci++) {
            if ($sampleLociMarkerGT->[$j_loci] ne '--') {
                $E_Gd_GaY                     = $array_E_Gd_GaY_hash[$j_loci]->{$sampleLociGaY->[$j_loci]};
                $sample_asum[$k_sample]      += $sampleLociGaValue->[$j_loci];
                $sample_a_each_locus[$j_loci] = $sampleLociGaValue->[$j_loci];
            } else {
                $E_Gd_GaY                     = $array_missingData_E_Gd[$j_loci]->{'case'};
                $sample_asum[$k_sample]      += 0;
                $sample_a_each_locus[$j_loci] = '-';
            }
            $sample_EDsum[$k_sample]           += $E_Gd_GaY;
            $sampleMarkerGT_each_locus[$j_loci] = $sampleLociMarkerGT->[$j_loci];
            $sample_ED_each_locus[$j_loci]      = $E_Gd_GaY;
            #print "    j_loci: $j_loci\t$sampleLociGaY->[$j_loci]\t $E_Gd_GaY\n";
        }
        $sample_control0[$k_sample] = $sample_control0;
        $sample_case1[$k_sample]    = $sample_case1;
        $sampleMarkerGT[$k_sample]  = \@sampleMarkerGT_each_locus;
        $sample_a[$k_sample]        = \@sample_a_each_locus;
        $sample_ED[$k_sample]       = \@sample_ED_each_locus;
        $sample_ID[$k_sample]       = $caseID_AX->[$i];
        #print "  dataset ",$i_dataset+1," (individul ",$k_sample+1,") ED: $sample_EDsum[$k_sample] D: $sample_dsum[$k_sample]\n";
        $k_sample++;
    }
    
    ####################### write individual ED file #######################
    my $wf = $paramsFileTemp_AX;
    open (W,">$wf") || die "$!";
    print W "sampleID\tisCase\ta_sum\tED_sum";
    for ($i=0;$i<$lociNumber_AX;$i++) {
        print W "\ta_",$i+1,"\tED_",$i+1;
    }
    print W "\n";
    for ($i=0;$i<scalar(@sample_ED);$i++) {
        print W $sample_ID[$i],"\t";
        print W $sample_case1[$i],"\t";
        print W $sample_asum[$i],"\t";
        print W $sample_EDsum[$i];
        for ($j=0;$j<$lociNumber_AX;$j++) {
            print W "\t",$sample_a[$i]->[$j];
            print W "\t",$sample_ED[$i]->[$j];
        }
        print W "\n";
    }
    close W;
    ####################### write individual ED file #######################
}

sub calculateHaplotypeP {
    my ($p_d_HP,$p_A_HP,$p_a_HP,$lociNumber_HP) = @_;
    my (@dA_HP,@DA_HP,@Da_HP,@da_HP);
    my ($i);
    for ($i=0;$i<$lociNumber_HP;$i++) {
        $dA_HP[$i] = 0;
        $DA_HP[$i] = $p_A_HP->[$i];
        $Da_HP[$i] = $p_a_HP->[$i]-$p_d_HP->[$i];
        $da_HP[$i] = $p_d_HP->[$i];
        print "locus",$i+1,":\n";
        print "  p_dA: ",$dA_HP[$i],"\tp_DA: ",$DA_HP[$i],"\tp_Da: ",$Da_HP[$i],"\tp_da: ",$da_HP[$i],"\n";
    }
    print "\n";
    return (\@dA_HP,\@DA_HP,\@Da_HP,\@da_HP);
}

sub calculateE_Gd_GaY {
    my ($K_CE,$p_Ga_Y0_CE,$p_Ga_Y1_CE,$p_dA_CE,$p_DA_CE,$p_Da_CE,$p_da_CE) = @_;
    my ($key);
#print "  E_Gd_GaY:\n";
#print "  p_{dA}: $p_dA_CE (0)\t    p_{DA}: $p_DA_CE (p_A)\t    p_{Da}: $p_Da_CE (p_a-p_d)\t    p_{da}: $p_da_CE (p_d)\n";
    
    my (%Ga);
    $Ga{'AA'} = $p_Ga_Y1_CE->{'AA'}*$K_CE + $p_Ga_Y0_CE->{'AA'}*(1-$K_CE);
    $Ga{'Aa|aA'} = $p_Ga_Y1_CE->{'Aa|aA'}*$K_CE + $p_Ga_Y0_CE->{'Aa|aA'}*(1-$K_CE);
    $Ga{'aa'} = $p_Ga_Y1_CE->{'aa'}*$K_CE + $p_Ga_Y0_CE->{'aa'}*(1-$K_CE);
#print "  P(Ga=i):\t    Ga(0): $Ga{'AA'} (AA)\t    Ga(1): $Ga{'Aa|aA'} (Aa|aA)\t    Ga(2): $Ga{'aa'} (aa)\n";
    
#    my ($g_0,$g_1,$g_2);
#    $g_0 = $K_CE*$p_Ga_Y1_CE->{'AA'}/$Ga{'AA'};
#    $g_1 = $K_CE*$p_Ga_Y1_CE->{'Aa|aA'}/$Ga{'Aa|aA'};
#    $g_2 = $K_CE*$p_Ga_Y1_CE->{'aa'}/$Ga{'aa'};
#print "  P(Y=1|Ga=i) = g_i\n    g_0: $g_0\n    g_1: $g_1\n    g_2: $g_2\n";

    my (%GdGa);
    $GdGa{'DDAA'} = $p_DA_CE**2;
    $GdGa{'DdAA'} = 2*$p_DA_CE*$p_dA_CE;
    $GdGa{'ddAA'} = $p_dA_CE**2;
    $GdGa{'DDAa'} = 2*$p_DA_CE*$p_Da_CE;
    $GdGa{'DdAa'} = 2*$p_DA_CE*$p_da_CE+2*$p_Da_CE*$p_dA_CE;
    $GdGa{'ddAa'} = 2*$p_dA_CE*$p_da_CE;
    $GdGa{'DDaa'} = $p_Da_CE**2;
    $GdGa{'Ddaa'} = 2*$p_Da_CE*$p_da_CE;
    $GdGa{'ddaa'} = $p_da_CE**2;
#print "  P(Gd,Ga)\n";
     foreach $key (sort keys %GdGa) {
#print "    GdGa{$key}: $GdGa{$key}\n";
     }
    
    my ($P_Y1Ga0,$P_Y1Ga1,$P_Y1Ga2);
#    $P_Y1Ga0 = $g_0*$Ga{'AA'};
#    $P_Y1Ga1 = $g_1*$Ga{'Aa|aA'};
#    $P_Y1Ga2 = $g_2*$Ga{'aa'};
    
    $P_Y1Ga0 = $K_CE*$p_Ga_Y1_CE->{'AA'};
    $P_Y1Ga1 = $K_CE*$p_Ga_Y1_CE->{'Aa|aA'};
    $P_Y1Ga2 = $K_CE*$p_Ga_Y1_CE->{'aa'};
#print "  P(Y=1,Ga=0)\n    P_Y1Ga0: $P_Y1Ga0\n    P_Y1Ga1: $P_Y1Ga1\n    P_Y1Ga2: $P_Y1Ga2\n";
    
    my ($f_0,$f_1,$f_2);
    # linear equations:
    # $P_Y1Ga0 = $f_0*$GdGa{'DDAA'} + $f_1*$GdGa{'DdAA'} + $f_2*$GdGa{'ddAA'};
    # $P_Y1Ga1 = $f_0*$GdGa{'DDAa'} + $f_1*$GdGa{'DdAa'} + $f_2*$GdGa{'ddAa'};
    # $P_Y1Ga2 = $f_0*$GdGa{'DDaa'} + $f_1*$GdGa{'Ddaa'} + $f_2*$GdGa{'ddaa'};
    
    ##################### solve $f_0,$f_1,$f_2 #####################
    my ($A,$b,$x,$B,$dim,$test,$LR,$string_matrix);
    my ($A_,$b_);
    $string_matrix = "[ $GdGa{'DDAA'} $GdGa{'DdAA'} $GdGa{'ddAA'} ]\n[ $GdGa{'DDAa'} $GdGa{'DdAa'} $GdGa{'ddAa'} ]\n[ $GdGa{'DDaa'} $GdGa{'Ddaa'} $GdGa{'ddaa'} ]\n";
    $A = Math::MatrixReal->new_from_string($string_matrix);
    $string_matrix = "[ $P_Y1Ga0 ]\n[ $P_Y1Ga1 ]\n[ $P_Y1Ga2 ]\n";
    $b = Math::MatrixReal->new_from_string($string_matrix);
#print "b=\n",$b;
#print "A=\n",$A;
    $LR = $A->decompose_LR();
    if (($dim,$x,$B) = $LR->solve_LR($b))
    {
        $test = $A * $x;
    }
    ($A_,$b_) = $A->normalize($b);
    $LR = $A_->decompose_LR();
    if (($dim,$x,$B) = $LR->solve_LR($b_))
    {
        $test = $A * $x;
#print "x = \n",$x;
#print "A * x = b\n",$test;
    }
    $f_0 = $x->element(1,1);
    $f_1 = $x->element(2,1);
    $f_2 = $x->element(3,1);
    
    if ($f_1 < 0) {
        $f_1 = $f_0;
        if ($f_2 < 0) {
            $f_2 = $f_1;
        }
    }
    elsif ($f_2 < 0) {
        $f_2 = $f_1;
    }
print "  P(Y=1|Gd=i)/f_i\n    f_0: $f_0 f_1: $f_1 f_2: $f_2\n";
    ##################### solve $f_0,$f_1,$f_2 #####################

    my (%P_GdGaY);
    $P_GdGaY{'DDAA1'} = $f_0*$GdGa{'DDAA'};
    $P_GdGaY{'DdAA1'} = $f_1*$GdGa{'DdAA'};
    $P_GdGaY{'ddAA1'} = $f_2*$GdGa{'ddAA'};
    $P_GdGaY{'DDAa1'} = $f_0*$GdGa{'DDAa'};
    $P_GdGaY{'DdAa1'} = $f_1*$GdGa{'DdAa'};
    $P_GdGaY{'ddAa1'} = $f_2*$GdGa{'ddAa'};
    $P_GdGaY{'DDaa1'} = $f_0*$GdGa{'DDaa'};
    $P_GdGaY{'Ddaa1'} = $f_1*$GdGa{'Ddaa'};
    $P_GdGaY{'ddaa1'} = $f_2*$GdGa{'ddaa'};    
    $P_GdGaY{'DDAA0'} = (1-$f_0)*$GdGa{'DDAA'};
    $P_GdGaY{'DdAA0'} = (1-$f_1)*$GdGa{'DdAA'};
    $P_GdGaY{'ddAA0'} = (1-$f_2)*$GdGa{'ddAA'};
    $P_GdGaY{'DDAa0'} = (1-$f_0)*$GdGa{'DDAa'};
    $P_GdGaY{'DdAa0'} = (1-$f_1)*$GdGa{'DdAa'};
    $P_GdGaY{'ddAa0'} = (1-$f_2)*$GdGa{'ddAa'};
    $P_GdGaY{'DDaa0'} = (1-$f_0)*$GdGa{'DDaa'};
    $P_GdGaY{'Ddaa0'} = (1-$f_1)*$GdGa{'Ddaa'};
    $P_GdGaY{'ddaa0'} = (1-$f_2)*$GdGa{'ddaa'};
#print "  P(Gd,Ga,Y)\n";
    foreach $key (sort keys %P_GdGaY) {
#print "    P_GdGaY{$key}: $P_GdGaY{$key}\n";
    }
    
    my (%P_GaY);
    $P_GaY{'AA1'} = $P_GdGaY{'DDAA1'}+$P_GdGaY{'DdAA1'}+$P_GdGaY{'ddAA1'};
    $P_GaY{'Aa1'} = $P_GdGaY{'DDAa1'}+$P_GdGaY{'DdAa1'}+$P_GdGaY{'ddAa1'};
    $P_GaY{'aa1'} = $P_GdGaY{'DDaa1'}+$P_GdGaY{'Ddaa1'}+$P_GdGaY{'ddaa1'};
    $P_GaY{'AA0'} = $P_GdGaY{'DDAA0'}+$P_GdGaY{'DdAA0'}+$P_GdGaY{'ddAA0'};
    $P_GaY{'Aa0'} = $P_GdGaY{'DDAa0'}+$P_GdGaY{'DdAa0'}+$P_GdGaY{'ddAa0'};
    $P_GaY{'aa0'} = $P_GdGaY{'DDaa0'}+$P_GdGaY{'Ddaa0'}+$P_GdGaY{'ddaa0'};
print "  P(Ga,Y)\n";
    foreach $key (sort keys %P_GaY) {
print "    P_GaY{$key}: $P_GaY{$key}\n";
    }
    
    my (%P_Gd_GaY);
    $P_Gd_GaY{'DD|AA1'} = $P_GdGaY{'DDAA1'}/$P_GaY{'AA1'};
    $P_Gd_GaY{'Dd|AA1'} = $P_GdGaY{'DdAA1'}/$P_GaY{'AA1'};
    $P_Gd_GaY{'dd|AA1'} = $P_GdGaY{'ddAA1'}/$P_GaY{'AA1'};
    $P_Gd_GaY{'DD|Aa1'} = $P_GdGaY{'DDAa1'}/$P_GaY{'Aa1'};
    $P_Gd_GaY{'Dd|Aa1'} = $P_GdGaY{'DdAa1'}/$P_GaY{'Aa1'};
    $P_Gd_GaY{'dd|Aa1'} = $P_GdGaY{'ddAa1'}/$P_GaY{'Aa1'};
    $P_Gd_GaY{'DD|aa1'} = $P_GdGaY{'DDaa1'}/$P_GaY{'aa1'};
    $P_Gd_GaY{'Dd|aa1'} = $P_GdGaY{'Ddaa1'}/$P_GaY{'aa1'};
    $P_Gd_GaY{'dd|aa1'} = $P_GdGaY{'ddaa1'}/$P_GaY{'aa1'};    
    $P_Gd_GaY{'DD|AA0'} = $P_GdGaY{'DDAA0'}/$P_GaY{'AA0'};
    $P_Gd_GaY{'Dd|AA0'} = $P_GdGaY{'DdAA0'}/$P_GaY{'AA0'};
    $P_Gd_GaY{'dd|AA0'} = $P_GdGaY{'ddAA0'}/$P_GaY{'AA0'};
    $P_Gd_GaY{'DD|Aa0'} = $P_GdGaY{'DDAa0'}/$P_GaY{'Aa0'};
    $P_Gd_GaY{'Dd|Aa0'} = $P_GdGaY{'DdAa0'}/$P_GaY{'Aa0'};
    $P_Gd_GaY{'dd|Aa0'} = $P_GdGaY{'ddAa0'}/$P_GaY{'Aa0'};
    $P_Gd_GaY{'DD|aa0'} = $P_GdGaY{'DDaa0'}/$P_GaY{'aa0'};
    $P_Gd_GaY{'Dd|aa0'} = $P_GdGaY{'Ddaa0'}/$P_GaY{'aa0'};
    $P_Gd_GaY{'dd|aa0'} = $P_GdGaY{'ddaa0'}/$P_GaY{'aa0'};
#print "  P(Gd|Ga,Y)\n";
    foreach $key (sort keys %P_Gd_GaY) {
#print "    P_Gd_GaY{$key}: $P_Gd_GaY{$key}\n";
    }
    
    my (%E_Gd_GaY);
    $E_Gd_GaY{'AA1'} = $P_Gd_GaY{'Dd|AA1'} + 2*$P_Gd_GaY{'dd|AA1'};
    $E_Gd_GaY{'Aa1'} = $P_Gd_GaY{'Dd|Aa1'} + 2*$P_Gd_GaY{'dd|Aa1'};
    $E_Gd_GaY{'aa1'} = $P_Gd_GaY{'Dd|aa1'} + 2*$P_Gd_GaY{'dd|aa1'};
    $E_Gd_GaY{'AA0'} = $P_Gd_GaY{'Dd|AA0'} + 2*$P_Gd_GaY{'dd|AA0'};
    $E_Gd_GaY{'Aa0'} = $P_Gd_GaY{'Dd|Aa0'} + 2*$P_Gd_GaY{'dd|Aa0'};
    $E_Gd_GaY{'aa0'} = $P_Gd_GaY{'Dd|aa0'} + 2*$P_Gd_GaY{'dd|aa0'};
print "  E(Gd|Ga,Y)\n";
    foreach $key (sort keys %E_Gd_GaY) {
print "    E_Gd_GaY{$key}: $E_Gd_GaY{$key}\n";
    }
    
    my ($missingData_E_Gd_Y1,$missingData_E_Gd_Y0,%missingData_E_Gd);
    $missingData_E_Gd_Y1         = $E_Gd_GaY{'AA1'}*$P_GaY{'AA1'}
                                 + $E_Gd_GaY{'Aa1'}*$P_GaY{'Aa1'}
                                 + $E_Gd_GaY{'aa1'}*$P_GaY{'aa1'};
    $missingData_E_Gd_Y0         = $E_Gd_GaY{'AA0'}*$P_GaY{'AA0'}
                                 + $E_Gd_GaY{'Aa0'}*$P_GaY{'Aa0'}
                                 + $E_Gd_GaY{'aa0'}*$P_GaY{'aa0'};
    $missingData_E_Gd{'case'}    = $missingData_E_Gd_Y1;
    $missingData_E_Gd{'control'} = $missingData_E_Gd_Y0;

print "  missingData E_Gd: $missingData_E_Gd_Y1 (case)\t$missingData_E_Gd_Y0 (control)\n";
    
    return (\%E_Gd_GaY,\%missingData_E_Gd);
}

# to suit S1.R, 'a' presencts marker no mater marker is minor or major
sub getSampleLociGaY {
    my ($ccLociMarkerSnpGT_GaY,$cc_GaY) = @_;
    my (@lociGaY,@lociGaValue,@lociMarkerGT,$i,$genotype_GaY);
    my ($sample_case1_GaY,$sample_control0_GaY);
    #control 0, case 1
    if ($cc_GaY == 0) {
        $sample_control0_GaY = 1;
        $sample_case1_GaY    = 0;
    }
    if ($cc_GaY == 1) {
        $sample_control0_GaY = 0;
        $sample_case1_GaY    = 1;
    }
    for ($i=0;$i<scalar(@$ccLociMarkerSnpGT_GaY);$i++) {
        $genotype_GaY = $ccLociMarkerSnpGT_GaY->[$i];
        if ($genotype_GaY eq 'Aa') {
            $lociGaY[$i]      = "Aa".$cc_GaY;
            $lociMarkerGT[$i] = "Aa";
            $lociGaValue[$i]  = 1;
        }
        if ($genotype_GaY eq 'aA') {
            $lociGaY[$i]      = "Aa".$cc_GaY;
            $lociMarkerGT[$i] = "aA";
            $lociGaValue[$i]  = 1;
        }
        if ($genotype_GaY eq 'aa') {
            $lociGaY[$i]      = "aa".$cc_GaY;
            $lociMarkerGT[$i] = "aa";
            $lociGaValue[$i]  = 2;
        }
        if ($genotype_GaY eq 'AA') {
            $lociGaY[$i]      = "AA".$cc_GaY;
            $lociMarkerGT[$i] = "AA";
            $lociGaValue[$i]  = 0;
        }
        if ($genotype_GaY eq '--') {
            $lociGaY[$i]      = "--".$cc_GaY;
            $lociMarkerGT[$i] = "--";
            $lociGaValue[$i]  = '-';
        }
    }
    return (\@lociGaY,\@lociGaValue,\@lociMarkerGT,$sample_control0_GaY,$sample_case1_GaY);
}

